@@ -1,35 +1,37 @@
 # -*- perl -*-
 #
 #  Net::Server::PreForkSimple - Net::Server personality
-#  
-#  $Id: PreForkSimple.pm,v 1.4 2002/03/27 22:15:43 rhandom Exp $
-#  
-#  Copyright (C) 2001, Paul T Seamons
-#                      paul@seamons.com
-#                      http://seamons.com/
-#  
+#
+#  $Id: PreForkSimple.pm,v 1.17 2005/06/21 20:35:30 rhandom Exp $
+#
+#  Copyright (C) 2001-2005
+#
+#    Paul Seamons
+#    paul@seamons.com
+#    http://seamons.com/
+#
+#    Rob Brown bbb@cpan,org
+#
 #  This package may be distributed under the terms of either the
-#  GNU General Public License 
+#  GNU General Public License
 #    or the
 #  Perl Artistic License
 #
 #  All rights reserved.
-#  
+#
 ################################################################
 
 package Net::Server::PreForkSimple;
 
+use base qw(Net::Server);
 use strict;
-use vars qw($VERSION @ISA $LOCK_EX $LOCK_UN);
-use POSIX qw(WNOHANG);
+use vars qw($VERSION);
+use POSIX qw(WNOHANG EINTR);
 use Fcntl ();
-use Net::Server ();
 use Net::Server::SIG qw(register_sig check_sigs);
 
 $VERSION = $Net::Server::VERSION; # done until separated
 
-### fall back to parent methods
-@ISA = qw(Net::Server);
 
 ### override-able options for this package
 sub options {
@@ -81,7 +83,7 @@ sub post_bind {
   $self->SUPER::post_bind;
 
   ### clean up method to use for serialization
-  if( ! defined($prop->{serialize}) 
+  if( ! defined($prop->{serialize})
       || $prop->{serialize} !~ /^(flock|semaphore|pipe)$/i ){
     $prop->{serialize} = 'flock';
   }
@@ -108,7 +110,7 @@ sub post_bind {
                                 ) || $self->fatal("Semaphore error [$!]");
     $s->setall(1) || $self->fatal("Semaphore create error [$!]");
     $prop->{sem} = $s;
-    
+
   ### set up pipe
   }elsif( $prop->{serialize} eq 'pipe' ){
     pipe( _WAITING, _READY );
@@ -117,7 +119,7 @@ sub post_bind {
     $prop->{_READY}   = *_READY;
     $prop->{_WAITING} = *_WAITING;
     print _READY "First\n";
-    
+
   }else{
     $self->fatal("Unknown serialization type \"$prop->{serialize}\"");
   }
@@ -139,7 +141,7 @@ sub loop {
 
   ### finish the parent routines
   $self->run_parent;
-  
+
 }
 
 ### subroutine to start up a specified number of children
@@ -149,8 +151,10 @@ sub run_n_children {
   my $n     = shift;
   return unless $n > 0;
 
+  $self->run_n_children_hook;
+
   $self->log(3,"Starting \"$n\" children");
-  
+
   for( 1..$n ){
     my $pid = fork;
 
@@ -170,32 +174,37 @@ sub run_n_children {
   }
 }
 
+### let the parent have more accounting upon startup of children
+sub run_n_children_hook {}
+
 ### child process which will accept on the port
 sub run_child {
   my $self = shift;
   my $prop = $self->{server};
 
-  ### restore sigs (turn off warnings during)
-  $SIG{INT} = $SIG{TERM} = $SIG{QUIT}
-    = $SIG{CHLD} = $SIG{PIPE} = 'DEFAULT';
-
-  $self->log(4,"Child Preforked ($$)\n");
-
-  $self->child_init_hook;
-
-  ### let the parent shut me down
-  $prop->{connected} = 0;
-  $prop->{SigHUPed}  = 0;
-  $SIG{HUP} = sub {
-    unless( $prop->{connected} ){
+  $SIG{INT} = $SIG{TERM} = $SIG{QUIT} = sub {
+    $self->child_finish_hook;
+    exit;
+  };
+  $SIG{PIPE} = 'IGNORE';
+  $SIG{CHLD} = 'DEFAULT';
+  $SIG{HUP}  = sub {
+    if (! $prop->{connected}) {
+      $self->child_finish_hook;
       exit;
     }
     $prop->{SigHUPed} = 1;
   };
 
+  $self->log(4,"Child Preforked ($$)\n");
+
+  delete $prop->{children};
+
+  $self->child_init_hook;
+
   ### accept connections
   while( $self->accept() ){
-    
+
     $prop->{connected} = 1;
 
     $self->run_client_connection;
@@ -205,7 +214,7 @@ sub run_child {
     $prop->{connected} = 0;
 
   }
-  
+
   $self->child_finish_hook;
 
   $self->log(4,"Child leaving ($prop->{max_requests})");
@@ -234,8 +243,10 @@ sub accept {
   if( $prop->{serialize} eq 'flock' ){
     open(LOCK,">$prop->{lock_file}")
       || $self->fatal("Couldn't open lock file \"$prop->{lock_file}\" [$!]");
-    flock(LOCK,Fcntl::LOCK_EX())
-      || $self->fatal("Couldn't get lock on file \"$prop->{lock_file}\" [$!]");
+    while (! flock(LOCK, Fcntl::LOCK_EX())) {
+      next if $! == EINTR;
+      $self->fatal("Couldn't get lock on file \"$prop->{lock_file}\" [$!]");
+    }
 
   }elsif( $prop->{serialize} eq 'semaphore' ){
     $prop->{sem}->op( 0, -1, IPC::SysV::SEM_UNDO() )
@@ -272,6 +283,9 @@ sub accept {
 sub done {
   my $self = shift;
   my $prop = $self->{server};
+  $prop->{done} = shift if @_;
+  return 1 if $prop->{done};
+
   return 1 if $prop->{requests} >= $prop->{max_requests};
   return 1 if $prop->{SigHUPed};
   if( ! kill(0,$prop->{ppid}) ){
@@ -287,7 +301,7 @@ sub run_parent {
   my $prop = $self->{server};
 
   $self->log(4,"Parent ready for children.\n");
-  
+
   ### set some waypoints
   $prop->{last_checked_for_dead}
   = $prop->{last_checked_for_dequeue}
@@ -305,7 +319,7 @@ sub run_parent {
                    $self->delete_child($chld);
                  }
                },
-### uncomment this area to allow SIG USR1 to give some runtime debugging               
+### uncomment this area to allow SIG USR1 to give some runtime debugging
 #               USR1 => sub {
 #                 require "Data/Dumper.pm";
 #                 print Data::Dumper::Dumper($self);
@@ -362,13 +376,17 @@ sub run_parent {
       }
     }
 
-
   }
 
   ### allow fall back to main run method
 
 }
 
+### Stub function in case check_for_dequeue is used.
+sub run_dequeue {
+  die "run_dequeue: virtual method not defined";
+}
+
 1;
 
 __END__
@@ -380,7 +398,7 @@ Net::Server::PreForkSimple - Net::Server personality
 =head1 SYNOPSIS
 
   use Net::Server::PreForkSimple;
-  @ISA = qw(Net::Server::PreFork);
+  @ISA = qw(Net::Server::PreForkSimple);
 
   sub process_request {
      #...code...
@@ -411,7 +429,7 @@ Please see the sample listed in Net::Server.
 =head1 COMMAND LINE ARGUMENTS
 
 In addition to the command line arguments of the Net::Server
-base class, Net::Server::PreFork contains several other 
+base class, Net::Server::PreFork contains several other
 configurable parameters.
 
   Key               Value                   Default
@@ -421,7 +439,7 @@ configurable parameters.
   serialize         (flock|semaphore|pipe)  undef
   # serialize defaults to flock on multi_port or on Solaris
   lock_file         "filename"              POSIX::tmpnam
-                                            
+
   check_for_dead    \d+                     30
 
   max_dequeue       \d+                     undef
@@ -447,7 +465,7 @@ On multi_port servers or on servers running on Solaris, the
 default is flock.  The flock option uses blocking exclusive
 flock on the file specified in I<lock_file> (see below).
 The semaphore option uses IPC::Semaphore (thanks to Bennett
-Todd) for giving some sample code.  The pipe option reads on a 
+Todd) for giving some sample code.  The pipe option reads on a
 pipe to choose the next.  the flock option should be the
 most bulletproof while the pipe option should be the most
 portable.  (Flock is able to reliquish the block if the
@@ -476,11 +494,13 @@ be checked by the check_for_dead variable.
 
 =item check_for_dequeue
 
-Seconds to wait before forking off a dequeue process.  It
-is intended to use the dequeue process to take care of 
+Seconds to wait before forking off a dequeue process.  The
+run_dequeue hook must be defined when using this setting.
+It is intended to use the dequeue process to take care of
 items such as mail queues.  If a value of undef is given,
 no dequeue processes will be started.
 
+
 =back
 
 =head1 CONFIGURATION FILE
@@ -521,7 +541,7 @@ white space are ignored.
 
   ### reverse lookups ?
   # reverse_lookups on
- 
+
   #-------------- file test.conf --------------
 
 =head1 PROCESS FLOW
@@ -535,10 +555,19 @@ always C<max_servers> running.
 
 =head1 HOOKS
 
-There are two additional hooks in the PreForkSimple server.
+The PreForkSimple server has the following hooks in addition
+to the hooks provided by the Net::Server base class.
+See L<Net::Server>
 
 =over 4
 
+=item C<$self-E<gt>run_n_children_hook()>
+
+This hook occurs at the top of run_n_children which is called
+each time the server goes to start more child processes.  This
+gives the parent to do a little of its own accountting (as desired).
+Idea for this hook came from James FitzGibbon.
+
 =item C<$self-E<gt>child_init_hook()>
 
 This hook takes place immeditately after the child process
@@ -551,9 +580,14 @@ the most shared memory possible is used.
 =item C<$self-E<gt>child_finish_hook()>
 
 This hook takes place immediately before the child tells
-the parent that it is exiting.  It is intended for 
+the parent that it is exiting.  It is intended for
 saving out logged information or other general cleanup.
 
+=item C<$self-E<gt>run_dequeue()>
+
+This hook only gets called in conjuction with the
+check_for_dequeue setting.
+
 =back
 
 =head1 TO DO
